package org.sonatype.tycho.tpr.packed;

import java.io.File;
import java.util.List;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.resolver.ArtifactResolutionException;
import org.apache.maven.artifact.resolver.ArtifactResolutionRequest;
import org.apache.maven.artifact.resolver.ArtifactResolutionResult;
import org.apache.maven.artifact.resolver.ResolutionErrorHandler;
import org.apache.maven.execution.MavenSession;
import org.apache.maven.model.Dependency;
import org.apache.maven.project.MavenProject;
import org.apache.maven.repository.RepositorySystem;
import org.codehaus.plexus.archiver.UnArchiver;
import org.codehaus.plexus.component.annotations.Component;
import org.codehaus.plexus.component.annotations.Requirement;
import org.codehaus.plexus.logging.Logger;
import org.codehaus.tycho.TargetPlatform;
import org.codehaus.tycho.TargetPlatformResolver;
import org.codehaus.tycho.osgitools.targetplatform.LocalTargetPlatformResolver;

@Component( role = TargetPlatformResolver.class, hint = PackedTargetPlatformResolver.ROLE_HINT, instantiationStrategy = "per-lookup" )
public class PackedTargetPlatformResolver
    extends LocalTargetPlatformResolver
{
    private static final String TYCHO_TARGET_PLATFORM_ARTIFACT = "tycho.targetPlatform.artifact";

    private static final String TYCHO_TARGET_PLATFORM_DEPENENCY = "tycho.targetPlatform.dependency";

    public static final String ROLE_HINT = "packed";

    @Requirement
    private RepositorySystem repositorySystem;

    @Requirement
    private ResolutionErrorHandler resolutionErrorHandler;

    @Requirement( hint = "zip" )
    private UnArchiver unArchiver;

    @Requirement
    private Logger logger;

    @Override
    public TargetPlatform resolvePlatform( final MavenSession session,
                                           final MavenProject project,
                                           final List<Dependency> dependencies )
    {
        final String gavDependency = project.getProperties().getProperty( TYCHO_TARGET_PLATFORM_DEPENENCY );
        final String gavArtifact = project.getProperties().getProperty( TYCHO_TARGET_PLATFORM_ARTIFACT );
        if ( gavArtifact == null && gavDependency == null )
        {
            throw new RuntimeException( String.format( "One of properties '%s' or '%s' must be specified", TYCHO_TARGET_PLATFORM_DEPENENCY, TYCHO_TARGET_PLATFORM_ARTIFACT ) );
        }
        Artifact artifact = null;
        if ( gavDependency != null )
        {
            logger.debug( String.format( "Using dependency '%s' as target platform", gavDependency ) );
            artifact = resolveDependency( session, project, createArtifactFromGav( gavDependency ) );
        }
        else
        {
            logger.debug( String.format( "Using artifact '%s' as target platform", gavArtifact ) );
            artifact = resolveArtifact( session, project, createArtifactFromGav( gavArtifact ) );
        }

        try
        {
            final File tempFile = File.createTempFile( "tycho-tpr-packed", "" );
            tempFile.delete();
            tempFile.mkdirs();
            final File location = new File( tempFile.getAbsolutePath(), "eclipse" );
            unArchiver.setSourceFile( artifact.getFile() );
            unArchiver.setDestDirectory( location.getParentFile() );
            unArchiver.extract();

            setLocation( location );

            return super.resolvePlatform( session, project, dependencies );
        }
        catch ( final Exception e )
        {
            throw new RuntimeException( e );
        }
    }

    private Artifact resolveDependency( final MavenSession session,
                                        final MavenProject project,
                                        final Artifact requested )
    {
        final List<Dependency> dependencies = project.getDependencies();
        if ( dependencies != null )
        {
            for ( final Dependency dependency : dependencies )
            {
                if ( ( requested.getGroupId().equals( "*" ) || dependency.getGroupId().equals( requested.getGroupId() ) )
                    && ( requested.getArtifactId().equals( "*" ) || dependency.getArtifactId().equals( requested.getArtifactId() ) )
                    && ( requested.getVersion().equals( "*" ) || dependency.getVersion().equals( requested.getVersion() ) )
                    && ( requested.getType().equals( "*" ) || dependency.getType().equals( requested.getType() ) )
                    && ( requested.getClassifier().equals( "*" ) || dependency.getClassifier().equals( requested.getClassifier() ) ) )
                {
                    return resolveArtifact( session, project, repositorySystem.createDependencyArtifact( dependency ) );
                }
            }
        }
        throw new RuntimeException( String.format( "There is no dependency matching '%s'", requested ) );
    }

    private Artifact resolveArtifact( final MavenSession session,
                                      final MavenProject project,
                                      final Artifact requested )
    {
        final ArtifactResolutionRequest request =
            new ArtifactResolutionRequest().setLocalRepository( session.getLocalRepository() )
                                           .setRemoteRepositories( project.getRemoteArtifactRepositories() )
                                           .setOffline( session.isOffline() )
                                           .setForceUpdate( session.getRequest().isUpdateSnapshots() )
                                           .setCache( session.getRepositoryCache() )
                                           .setServers( session.getRequest().getServers() )
                                           .setMirrors( session.getRequest().getMirrors() )
                                           .setProxies( session.getRequest().getProxies() )
                                           .setArtifact( requested )
                                           .setResolveTransitively( false );

        final ArtifactResolutionResult result = repositorySystem.resolve( request );
        try
        {
            resolutionErrorHandler.throwErrors( request, result );
        }
        catch ( final ArtifactResolutionException e )
        {
            throw new RuntimeException( e );
        }

        final Artifact resolved = result.getArtifacts().iterator().next();
        return resolved;
    }

    private Artifact createArtifactFromGav( final String gav )
    {
        final String[] segments = gav.split( ":" );

        return repositorySystem.createArtifactWithClassifier(
                                                              segments.length > 0 ? segments[0] : "*",
                                                              segments.length > 1 ? segments[1] : "*",
                                                              segments.length > 2 ? segments[2] : "*",
                                                              segments.length > 3 ? segments[3] : "*",
                                                              segments.length > 4 ? segments[4] : "*" );
    }
}
